/***************************************************************************
 *   Copyright (C) 2005 by Erik Eriksen   *
 *   eriker@farmingdale.edu   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <stdlib.h>
#include "SDL.h"

using namespace std;



struct CSpriteFrame
{
  SDL_Surface *image;
  int pause;
};

class CSpriteBase
{
  public:
    int init(char *dir);
    CSpriteFrame *mAnim;
    int mBuilt, mNumFrames, mW, mH;
};

int CSpriteBase::init(char *dir)
{
  char buffer[255];
  char filename[255];
  char name[255];
  int pause = 0, r = 0, g = 0, b = 0;
  FILE *fp;

  sprintf(filename, "%s/info", dir);

  if ((fp = fopen(filename, "r")) = NULL)
  {
    perror("ERROR opening file");// %s\n\n", filename);
    return -1;
  }

  fgets(buffer, 255, fp);
  sscanf(buffer, "FILES: %d", &mNumFrames);

  mAnim = new CSpriteFrame[mNumFrames];

  mBuilt = 1; //to tell that class has been built.
  int count = 0; //to keep track of files being opened.

  while (!feof(fp) && count < mNumFrames)
  {
    fgets(buffer, 255, fp);
    if (buffer[0] != '#' && buffer[0] != '\r' && buffer[0] != '\0' && buffer[0] != '\n' && strlen(buffer) != 0)
    {
      sscanf(buffer, "%s %d %d %d %d", name, &pause, &r, &g, &b); //file loc, pause in miliseconds, transparency color in RGB
      sprintf(filename, "%s/%s", dir, name); //create full location of image file to load.

      SDL_Surface *temp; //temp surface to load image into
      if ((temp = SDL_LoadBMP(filename)) == NULL) //error handling for file loading.
        return -1;

      if (r > 0) //if greater than zero temp becomes transparent
        SDL_SetColorKey(temp, SDL_SRCCOLORKEY, SDL_MapRGB(temp->format, r, g, b));

      mAnim[count].image = SDL_DisplayFormat(temp); //take format of temp and put it to the animation array.
      SDL_FreeSurface(temp);

      mAnim[count].pause = pause;  //setting all the values to attributes of mAnim.
      if (!mW)
        mW = mAnim[count].image->w;
      if (!mH)
        mH = mAnim[count].image->h;

      count++; //file loading incriment.
    }
  }

  fclose(fp); //close the file.
  return 0; //no errors.
}

class CSprite
{
  public:
    int init(CSpriteBase *base, SDL_Surface *screen);
    void draw();
    void clearBG();
    void updateBG();
    void setFrame(int nr) {mFrame = nr;}
    int getFrame() {return mFrame;}
    void setSpeed(float nr) {mSpeed = nr;}
    float getSpeed() {return mSpeed;}
    void toggleAnim() {mAnimating = !mAnimating;}
    void startAnim() {mAnimating = 1;}
    void stopAnim() {mAnimating = 0;}
    void rewind() {mFrame = 0;}
    void xAdd(int nr) {mX += nr;}
    void yAdd(int nr) {mY += nr;}
    void xSet(int nr) {mX = nr;}
    void ySet(int nr) {mY = nr;}
    void set(int xx, int yy) {mX = xx; mY = yy;}

  private:
    int mFrame;
    int mX, mY, mOldX, mOldY;
    int mAnimating;
    int mDrawn;
    float mSpeed;
    long mLastupdate;
    CSpriteBase *mSpriteBase;
    SDL_Surface *mBackreplacement;
    SDL_Surface *mScreen;
};

int CSprite::init(CSpriteBase *base, SDL_Surface *screen)
{
  mSpriteBase = base;
  if (mSpriteBase->mBuilt) //make sure the surface is built first.
  {
    if (mSpriteBase->mNumFrames > 1) //if there's more than one frame then 
      mAnimating = 1; //animate.	
    mBackreplacement = SDL_DisplayFormat(mSpriteBase->mAnim[0].image); //first frame of the animation.
  }
  mScreen = screen;
}

void CSprite::clearBG()
{
  if (mDrawn == 1) //if it's drawn... WE CAN ERASE IT!
  {
    SDL_Rect dest;  //setting up exactly where to reblit the background.
    dest.x = mOldX;
    dest.y = mOldY;
    dest.w = mSpriteBase->mW;
    dest.h = mSpriteBase->mH;
    SDL_BlitSurface(mBackreplacement, NULL, mScreen, &dest); //...blitting...
  }
}

void CSprite::updateBG()
{
  SDL_Rect srcRect;
  srcRect.w = mSpriteBase->mW;
  srcRect.h = mSpriteBase->mH;
  srcRect.x = mX;
  srcRect.y = mY;
  mOldX = mX; // setting it up so that clearBG() knows where to clear.
  mOldY = mY; 
  SDL_BlitSurface(mScreen, &srcRect, mBackreplacement, NULL);
}

void CSprite::draw()
{
  if (mAnimating == 1) //if it's animating.
  {
    if (mLastupdate + mSpriteBase->mAnim[mFrame].pause * mSpeed < SDL_GetTicks())
    {
      mFrame++;
      if (mFrame > mSpriteBase->mNumFrames -1) 
        mFrame = 0;
      mLastupdate = SDL_GetTicks();
    }
  }
  if (mDrawn == 0)
    mDrawn = 1; //so ClearBG knows there's something to clear.

  SDL_Rect dest;
  dest.x = mX;
  dest.y = mY;
  SDL_BlitSurface(mSpriteBase->mAnim[mFrame].image, NULL, mScreen, &dest);
}
//end of class functions

SDL_Surface *imageLoad(char *file) //loads image
{
  SDL_Surface *temp1, *temp2;
  temp1 = SDL_LoadBMP(file);
  temp2 = SDL_DisplayFormat(temp1);
  SDL_FreeSurface(temp1);
  return temp2;
}

int initImages() //loads background images
{
  SDL_Surface *back = imageLoad("bg.bmp");
  return 0;
}

void drawIMG(SDL_Surface *img, int x, int y, SDL_Surface *screen) //draws image at given x, y location
{
  SDL_Rect dest;
  dest.x = x;
  dest.y = y;
  SDL_BlitSurface(img, NULL, screen, &dest);
}

void drawBG(SDL_Surface *back, SDL_Surface *screen) //fills the screen with a background
{
  drawIMG(back, 0, 0, screen); //draw to screen back at (0,0)
}
//global variables (for now)
  SDL_Surface *screen, *back;
  CSpriteBase vikingBase;
  CSpriteBase sunBase;
  CSprite vikings1; 
  CSprite vikings2;
  CSprite sun;
//global variables.
void drawScene() //draws all the sprites
{
  sun.clearBG();
  vikings1.clearBG();
  vikings2.clearBG();

  sun.updateBG();
  vikings1.updateBG();
  vikings2.updateBG();

  sun.draw();
  vikings1.draw();
  vikings2.draw();

  SDL_Flip(screen);
}


//main function

int main(int argc, char *argv[])
{

  int scrWidth = 0, scrHeight = 0;
  int scrSizeSelection;
  bool game = true;
  long int i = 0;

  Uint8 *keys;

 cout <<"Initializing SDL..." << endl;
  /* Initializes Audio and SDL_INIT_VIDEO for Video */
  if(SDL_Init(SDL_INIT_AUDIO | SDL_INIT_VIDEO)< 0) {
    cout <<"Could not initialize SDL:" << SDL_GetError() << endl;
       SDL_Quit();
  }

  else 
  {
    cout << "Audio & Video initialized correctly\n" << endl;;
  }

  printf("Select desired screen resolution:\n");
  printf(" 1) 640 x 480\n");
  printf(" 2) 800 x 600\n");
  printf(" 3) 1024 x 768\n");
  //cin >> scrSizeSelection;
  scrSizeSelection = 3;

  switch(scrSizeSelection)
  {
    case 1:
      scrWidth = 640;
      scrHeight = 480;
      break;

    case 2:
      scrWidth = 800;
      scrHeight = 600;
      break;

    case 3:
      scrWidth = 1024;
      scrHeight = 768;
      break;

    default:
      printf("ERROR: Invalid screen resolution!\nQuiting...\n");
      SDL_Quit();
      break;
  }


  SDL_Surface *screen;

  screen = SDL_SetVideoMode(scrWidth, scrHeight, 32, SDL_SWSURFACE );//| SDL_FULLSCREEN);// | HWPALETTE);
  if (screen == NULL)
  {
    printf("ERROR: Unable to set screen resolution to %dx%d! %s\n", scrWidth, scrHeight, SDL_GetError());
    return 1;
  }


  //initialize images
  vikingBase.init("includes/vikings");
  sunBase.init("includes/sun");

  sun.init(&sunBase, screen);
  sun.set(480, 50);
  sun.setSpeed(1);

  vikings1.init(&vikingBase, screen);
  vikings1.set(150, 300);
  vikings1.setSpeed(1);

  vikings2.init(&vikingBase, screen);
  vikings2.set(350, 300);
  vikings2.setSpeed(1.5);

  //hide mouse cursor
  SDL_ShowCursor(0);

  //init images
  initImages();
  drawBG(screen,screen);



  while (game == true)
  {
     SDL_Event event;

    while(SDL_PollEvent(&event))
    {
      if (event.type == SDL_QUIT)
        game = false;
      if (event.type == SDL_KEYDOWN)
      {
        if (event.key.keysym.sym == SDLK_ESCAPE)
          game = false;
      }
    }

    keys = SDL_GetKeyState(NULL);
    if (keys[SDLK_UP])
      {
        vikings1.yAdd(-1);
      }


    drawScene();
  }

  SDL_Quit();
}

